/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "Map.h"
#include "Vector.h"
#include "SrcInfo.h"

using namespace std;
using namespace FD;

class CvfdhCreateHist;
   
DECLARE_NODE(CvfdhCreateHist)
/*Node
 *
 * @name CvfdhCreateHist
 * @category FDOPENCV_HARK:Hist
 * @description CvfdhCreateHist
 *
 * @input_name IMAGEIN
 * @input_type any
 * @input_description Input image. The type should be CvfdImage or Map<int, ObjectRef>.
 *
 * @input_name SRCIN
 * @input_type Vector<ObjectRef>
 * @input_description Source location. A type of an element of the vector is SrcInfo.
 *
 * @output_name FEATOUT
 * @output_type Map<int,ObjectRef>
 * @output_description Output features. The ObjectRef should be Vector<float>.
 *
 END*/
      
class CvfdhCreateHist : public BufferedNode {

  int m_imageInID;
  int m_srcInID;
  int m_featOutID;
      
public:
  CvfdhCreateHist(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params), m_srcInID(-1)
  {
    m_imageInID = addInput("IMAGEIN");
    m_featOutID = addOutput("FEATOUT");
    inOrder = true;
  }      

  virtual int translateInput(string inputName) {
    if (inputName == "SRCIN") {
      m_srcInID = addInput(inputName);
      return m_srcInID;
    }
    else if (inputName == "IMAGEIN") {
      for (unsigned int i = 0; i < inputs.size(); i++) {
	if (inputs[i].name == inputName) {
	  return i;
	}
      }
    }
    else {throw new NodeException(this, inputName+ " is not supported.", __FILE__, __LINE__);}
  }
      
  void calculate(int output_id, int count, Buffer &out)
  {

    RCPtr<Map<int, ObjectRef> > outputMapPtr(new Map<int, ObjectRef>);
    (*(outputs[m_featOutID].buffer))[count] = outputMapPtr;


    RCPtr<Map<int, ObjectRef> > image_map(new Map<int, ObjectRef>);

    // Making the Map<id, IplImage*> for processing
    IplImage* img_in_tmp;
    ObjectRef in = getInput(m_imageInID, count);
    if (typeid(*in) == typeid(CvfdImage)) {
      RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);   
      if(imagePtr->getImage() == NULL){
	return;
      }
      img_in_tmp = cvCloneImage(imagePtr->getImage());
      CvfdImage* image= new CvfdImage(img_in_tmp);  
      (*image_map)[0] = image;
    }else if (typeid(*in) == typeid(Map<int, ObjectRef>)) {
      RCPtr<Map<int, ObjectRef> > imageMapPtr = getInput(m_imageInID, count);
      Map<int, ObjectRef>::iterator it = imageMapPtr->begin();
      for (; it != imageMapPtr->end(); ++it) {
        CvfdImage& imagePtr = object_cast<CvfdImage>(it->second);
	int store_flag = 1;
	if(m_srcInID != -1){
	  RCPtr<Vector<ObjectRef> > sources_ptr = getInput(m_srcInID, count);
	  const Vector<ObjectRef>& sources = *sources_ptr;
	  store_flag = 0;
	  for (int srci = 0; srci < sources.size(); srci++) {
	    RCPtr<SrcInfo> src = sources[srci];
	    if(src->id == it->first) store_flag = 1;
	  }
	}
	if(store_flag == 0) continue;
	img_in_tmp = cvCloneImage(imagePtr.getImage());
	CvfdImage* image= new CvfdImage(img_in_tmp);  
	(*image_map)[it->first] = image;
      }
    }

    // Main routine
    Map<int, ObjectRef>::iterator img_it = image_map->begin();
    for (; img_it != image_map->end(); ++img_it) {
      IplImage* img_in;      
      CvfdImage& imagePtr = object_cast<CvfdImage>(img_it->second);
      img_in = cvCloneImage(imagePtr.getImage());
    
      // ---------------------------------------------------------------
      // main routine : start

      // IplImage* img_out;

      IplImage *hsv;
      IplImage *h_plane, *s_plane, *v_plane;
      IplImage *planes[2];
      int h_bins = 30, s_bins = 32;
      int hist_size[] = { h_bins, s_bins };
      float h_ranges[] = { 0, 181 };
      float s_ranges[] = { 0, 256 };
      float *ranges[] = { h_ranges, s_ranges };
      // int scale = 10;
      // IplImage *hist_img = cvCreateImage (cvSize (h_bins * scale, s_bins * scale), 8, 3);
      CvHistogram *hist;
      float max_value = 0;
      int h, s;

      hsv = cvCreateImage (cvGetSize (img_in), 8, 3);
      planes[0] = h_plane = cvCreateImage (cvGetSize (img_in), 8, 1);
      planes[1] = s_plane = cvCreateImage (cvGetSize (img_in), 8, 1);
      v_plane = cvCreateImage (cvGetSize (img_in), 8, 1);

      cvCvtColor (img_in, hsv, CV_BGR2HSV);
      cvCvtPixToPlane (hsv, h_plane, s_plane, v_plane, 0);
      hist = cvCreateHist (2, hist_size, CV_HIST_ARRAY, ranges, 1);
      cvCalcHist (planes, hist, 0, 0);
      cvGetMinMaxHistValue (hist, 0, &max_value, 0, 0);
      // cvZero (hist_img);

      RCPtr<Vector<float> > map_contents(new Vector<float>);
      
      for (h = 0; h < h_bins; h++) {
	for (s = 0; s < s_bins; s++) {
	  float bin_val = cvQueryHistValue_2D (hist, h, s);
	  (*map_contents).push_back((float)bin_val);
	  // int intensity = cvRound (bin_val * 255 / max_value);
	  // cvRectangle (hist_img, cvPoint (h * scale, s * scale), cvPoint ((h + 1) * scale - 1, (s + 1) * scale - 1), CV_RGB (intensity, intensity, intensity), CV_FILLED);
	}
      }

      (*outputMapPtr)[img_it->first] = map_contents;    

      // Output Setting
      // img_out = cvCloneImage (hist_img);
      // main routine : finish
      // ---------------------------------------------------------------
         
      // CvfdImage* image= new CvfdImage(img_out);  
      // (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(image); 

      cvReleaseImage(&img_in);
      // cvReleaseImage(&img_out);
      cvReleaseImage(&s_plane);
      cvReleaseImage(&hsv);
      cvReleaseImage(&h_plane);
      cvReleaseImage(&v_plane);
      cvReleaseImage(&planes[0]);
      cvReleaseImage(&planes[1]);
      // cvReleaseImage(&hist_img);
	       
    }

    cvReleaseImage(&img_in_tmp);

  }
  
  NO_ORDER_NODE_SPEEDUP(CvfdhCreateHist)
};
