/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "SrcInfo.h"
#include "Vector.h"

using namespace std;
using namespace FD;

class CvfdhDrawSrcRoi;
   
DECLARE_NODE(CvfdhDrawSrcRoi)
/*Node
 *
 * @name CvfdhDrawSrcRoi
 * @category FDOPENCV_HARK:ROI
 * @description Output an image with source windows
 *
 * @input_name IMAGEIN
 * @input_description Input image
 * @input_type CvfdImage
 *
 * @input_name SRCIN
 * @input_type Vector<ObjectRef>
 * @input_description Source location. A type of an element of the vector is SrcInfo.
 *
 * @output_name IMAGEOUT
 * @output_description Output image 
 * @output_type CvfdImage
 * 
 * @parameter_name WINDOW_COLOUR
 * @parameter_type object
 * @parameter_value <Vector<int> 0 0 0>
 * @parameter_description RGB window colour setting [0 - 255]
 * 
 * @parameter_name WINDOW_THICKNESS
 * @parameter_type int
 * @parameter_value 1
 * @parameter_description Window line thickness
 * 
 * @parameter_name CENTER_COLOUR
 * @parameter_type object
 * @parameter_value <Vector<int> 0 0 0>
 * @parameter_description RGB center point colour setting [0 - 255]
 * 
 * @parameter_name CENTER_THICKNESS
 * @parameter_type int
 * @parameter_value 1
 * @parameter_description Center point line thickness
 *
 END*/
   
   
class CvfdhDrawSrcRoi : public BufferedNode {
      
  int m_imageInID;
  int m_srcInID;
  int m_imageOutID;
  Vector<int> window_colour;
  Vector<int> center_colour;
  int window_thickness;
  int center_thickness;
            
public:
  CvfdhDrawSrcRoi(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    m_imageInID = addInput("IMAGEIN");
    m_srcInID = addInput("SRCIN");
    m_imageOutID = addOutput("IMAGEOUT");         
    window_colour = object_cast<Vector<int> >(parameters.get("WINDOW_COLOUR"));
    center_colour = object_cast<Vector<int> >(parameters.get("CENTER_COLOUR"));
    window_thickness = dereference_cast<int>(parameters.get("WINDOW_THICKNESS"));
    center_thickness = dereference_cast<int>(parameters.get("CENTER_THICKNESS"));    
    inOrder = true;
  }
      
  ~CvfdhDrawSrcRoi()
  {
  }
      
  void calculate(int output_id, int count, Buffer &out)
  {
    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);
    if(imagePtr->getImage() == NULL){
      (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(imagePtr);
      return;
    }

    IplImage* img_in;
    img_in = cvCloneImage(imagePtr->getImage());

    RCPtr<Vector<ObjectRef> > pcur_src = getInput(m_srcInID, count);
    const Vector<ObjectRef>& cur_sources = *pcur_src;

    for (int i = 0; i < cur_sources.size(); i++) {
      RCPtr<SrcInfo> src = cur_sources[i];
      float roi_c_x  = src->roi_c[0];
      float roi_c_y  = src->roi_c[1];
      float roi_xy_x = src->roi_xy[0];
      float roi_xy_y = src->roi_xy[1];
      float roi_min_x = roi_c_x - roi_xy_x / 2;
      float roi_min_y = roi_c_y - roi_xy_y / 2;
      float roi_max_x = roi_c_x + roi_xy_x / 2;
      float roi_max_y = roi_c_y + roi_xy_y / 2;

      cvLine( img_in, cvPoint(roi_c_x, roi_c_y), 
	      cvPoint(roi_c_x, roi_c_y), CV_RGB((int)center_colour[0], (int)center_colour[1], (int)center_colour[2]), (int)center_thickness, 8, 0 );
      cvRectangle( img_in, cvPoint(roi_min_x , roi_min_y), 
		   cvPoint ( roi_max_x, roi_max_y ), CV_RGB((int)window_colour[0], (int)window_colour[1], (int)window_colour[2]), (int)window_thickness, 8, 0);      
    }
        
    IplImage* img_out;
    img_out = cvCloneImage(img_in);

    CvfdImage* image= new CvfdImage(img_out);  
         
    (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(image); 

    cvReleaseImage (&img_in);
    cvReleaseImage (&img_out);

  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdhDrawSrcRoi)
};
