/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "SrcInfo.h"
#include "Vector.h"
#include "Map.h"
#include <vector>

using namespace std;
using namespace FD;

class CvfdhFindContours;
   
DECLARE_NODE(CvfdhFindContours)
/*Node
 *
 * @name CvfdhFindContours
 * @category FDOPENCV_HARK:Segmentation
 * @description CvfdhFindContours
 *
 * @input_name IMAGEIN
 * @input_type any
 * @input_description Input image. The type should be CvfdImage or Map<int, ObjectRef>.
 *
 * @output_name SRCOUT
 * @output_type Vector<ObjectRef>
 * @output_description Source location. A type of an element of the vector is SrcInfo.
 *
 END*/
   
   
class CvfdhFindContours : public BufferedNode {

  int m_imageInID;
  int m_srcOutID;
      
public:
  CvfdhFindContours(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    m_imageInID = addInput("IMAGEIN");
    m_srcOutID = addOutput("SRCOUT");
    inOrder = true;
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {

    Vector<ObjectRef>& sources = *new Vector<ObjectRef>;
    (*(outputs[m_srcOutID].buffer))[count] = &sources;


    vector<IplImage*> image_vec;
    image_vec.resize(0);

    IplImage* img_in_tmp;
    ObjectRef in = getInput(m_imageInID, count);
    if (typeid(*in) == typeid(CvfdImage)) {
      RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);   
      if(imagePtr->getImage() == NULL){
	return;
      }
      img_in_tmp = cvCloneImage(imagePtr->getImage());
      image_vec.push_back(img_in_tmp);
    }else if (typeid(*in) == typeid(Map<int, ObjectRef>)) {
      RCPtr<Map<int, ObjectRef> > imageMapPtr = getInput(m_imageInID, count);
      Map<int, ObjectRef>::iterator it = imageMapPtr->begin();
      for (; it != imageMapPtr->end(); ++it) {
        CvfdImage& imagePtr = object_cast<CvfdImage>(it->second);
	img_in_tmp = cvCloneImage(imagePtr.getImage());
	image_vec.push_back(img_in_tmp);
      }
    }

    for(int cnt = 0; cnt < image_vec.size(); cnt++){
      IplImage* img_in;      
      img_in = cvCloneImage((IplImage*)image_vec[cnt]);
    
      // ---------------------------------------------------------------
      // main routine : start

      IplImage *dst_img;
      IplImage *src_img_gray = 0;
      IplImage *tmp_img;
      CvMemStorage *storage = cvCreateMemStorage (0);
      CvSeq *contours = 0;
      int levels = 0;
      CvPoint *point;
      CvTreeNodeIterator it;
    
      src_img_gray = cvCreateImage (cvGetSize (img_in), IPL_DEPTH_8U, 1);
      cvCvtColor (img_in, src_img_gray, CV_BGR2GRAY);
      tmp_img = cvCreateImage (cvGetSize (img_in), IPL_DEPTH_8U, 1);
      dst_img = cvCloneImage (img_in);
    
      // Ͳ
      cvThreshold (src_img_gray, tmp_img, 120, 255, CV_THRESH_BINARY);
    
      // سԤθ
      cvFindContours (tmp_img, storage, &contours, sizeof (CvContour), CV_RETR_TREE, CV_CHAIN_APPROX_SIMPLE);
    
      cvInitTreeNodeIterator (&it, contours, 1);
    
      while ((contours = (CvSeq *) cvNextTreeNode (&it)) != NULL) {
      
	vector<int> roi_c;
	roi_c.resize(2);
	vector<int> roi_max;
	roi_max.resize(2);
	for(int cnt = 0; cnt < roi_max.size(); cnt++) roi_max[cnt] = 0;
	vector<int> roi_min;
	roi_min.resize(2);
	for(int cnt = 0; cnt < roi_min.size(); cnt++) roi_min[cnt] = 100000;

	for (int i = 0; i < contours->total; i++) {
	  point = CV_GET_SEQ_ELEM (CvPoint, contours, i);
	  // cvLine (cv_image, *tmp, *point, CV_RGB (0, 0, 255), 2);
	  roi_c[0] += point->x;
	  roi_c[1] += point->y;
	  if(roi_max[0] < point->x) roi_max[0] = point->x;
	  if(roi_max[1] < point->y) roi_max[1] = point->y;
	  if(roi_min[0] > point->x) roi_min[0] = point->x;
	  if(roi_min[1] > point->y) roi_min[1] = point->y;	
	}
	roi_c[0] /= contours->total;
	roi_c[1] /= contours->total;      

	RCPtr<SrcInfo> src(new SrcInfo);
	src->roi_c[0] = roi_c[0];
	src->roi_c[1] = roi_c[1];
	src->roi_xy[0] = roi_max[0] - roi_min[0];
	src->roi_xy[1] = roi_max[1] - roi_min[1];
	src->img_size[0] = img_in->width;
	src->img_size[1] = img_in->height;
      
	sources.push_back(src);

      }

      cvReleaseImage (&img_in);
      cvReleaseImage (&src_img_gray);
      cvReleaseImage (&dst_img);
      cvReleaseImage (&tmp_img);
      
    }

    cvReleaseImage(&img_in_tmp);

  }

      
  NO_ORDER_NODE_SPEEDUP(CvfdhFindContours)
};
