/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "SrcInfo.h"
#include "Source.h"
#include "CvfdCameraSpec.h"
#include "Vector.h"
#include "Map.h"

using namespace std;
using namespace FD;

class CvfdhSrcInfoMapping;
   
DECLARE_NODE(CvfdhSrcInfoMapping)
/*Node
 *
 * @name CvfdhSrcInfoMapping
 * @category FDOPENCV_HARK:ROI
 * @description CvfdhSrcInfoMapping
 *
 * @input_name SRCINFO_FROM
 * @input_type Vector<ObjectRef>
 * @input_description Source location before mapping. A type of an element of the vector is SrcInfo.
 *
 * @input_name CAMSPEC_FROM
 * @input_type CvfdCameraSpec
 * @input_description Camera specification of original image
 *
 * @input_name CAMSPEC_TO
 * @input_type CvfdCameraSpec
 * @input_description Camera specification of mapped image
 *
 * @output_name SRCINFO_TO
 * @output_type Vector<ObjectRef>
 * @output_description Source location after mapping. A type of an element of the vector is SrcInfo.
 *
 END*/
   
   
class CvfdhSrcInfoMapping : public BufferedNode {

  int m_camspec_from_ID;
  int m_srcinfo_from_ID;
  int m_camspec_to_ID;
  int m_srcinfo_to_ID;
      
public:
  CvfdhSrcInfoMapping(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    m_camspec_from_ID = addInput("CAMSPEC_FROM");
    m_srcinfo_from_ID = addInput("SRCINFO_FROM");
    m_camspec_to_ID = addInput("CAMSPEC_TO");
    m_srcinfo_to_ID = addOutput("SRCINFO_TO");
    inOrder = true;
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {

    RCPtr<CvfdCameraSpec> cspecfrom  = getInput(m_camspec_from_ID, count);
    RCPtr<CvfdCameraSpec> cspecto    = getInput(m_camspec_to_ID, count);

    RCPtr<Vector<ObjectRef> > srcinfo_from  = getInput(m_srcinfo_from_ID, count);

    Vector<ObjectRef>& srcinfo_to = *new Vector<ObjectRef>;
    (*(outputs[m_srcinfo_to_ID].buffer))[count] = &srcinfo_to;

    for (int i = 0; i < srcinfo_from->size(); i++) {

      RCPtr<SrcInfo> srci = (*srcinfo_from)[i];

      int ltpx_i = srci->roi_c[0] - srci->roi_xy[0] / 2;
      int ltpy_i = srci->roi_c[1] - srci->roi_xy[1] / 2;      
      int rbpx_i = srci->roi_c[0] + srci->roi_xy[0] / 2;
      int rbpy_i = srci->roi_c[1] + srci->roi_xy[1] / 2;      

      float ltradx_i = (*cspecfrom).pt2rad_x(ltpx_i);
      float ltrady_i = (*cspecfrom).pt2rad_y(ltpy_i);
      float rbradx_i = (*cspecfrom).pt2rad_x(rbpx_i);
      float rbrady_i = (*cspecfrom).pt2rad_y(rbpy_i);

      int ltpx_f = (*cspecto).rad2pt_x(ltradx_i);
      int ltpy_f = (*cspecto).rad2pt_y(ltrady_i);
      int rbpx_f = (*cspecto).rad2pt_x(rbradx_i);
      int rbpy_f = (*cspecto).rad2pt_y(rbrady_i);

      ltpx_f = min(max(ltpx_f,0),(*cspecto).width-1);
      ltpy_f = min(max(ltpy_f,0),(*cspecto).height-1);
      rbpx_f = min(max(rbpx_f,0),(*cspecto).width-1);
      rbpy_f = min(max(rbpy_f,0),(*cspecto).height-1);
      
      RCPtr<SrcInfo> srco(new SrcInfo);
      srco->roi_c[0] = (rbpx_f + ltpx_f) / 2;
      srco->roi_c[1] = (rbpy_f + ltpy_f) / 2;
      srco->roi_xy[0] = rbpx_f - ltpx_f;
      srco->roi_xy[1] = rbpy_f - ltpy_f;
      srco->img_size[0] = (*cspecto).width;
      srco->img_size[1] = (*cspecto).height;
      srcinfo_to.push_back(srco);

    }
  } 
  NO_ORDER_NODE_SPEEDUP(CvfdhSrcInfoMapping)
};
