/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "CvfdColor.h"

using namespace std;
using namespace FD;

class CvfdFindColorRGB;
   
DECLARE_NODE(CvfdFindColorRGB)
/*Node
 *
 * @name CvfdFindColorRGB
 * @category FDOPENCV_ORG:Color
 * @description Detects the color in the image with a RGB method
 * dst(I) is set to 0xff (all '1'-bits) if the color is detect and 0 otherwise.
 *
 * @input_name IMAGEIN
 * @input_description Input image
 * @input_type CvfdImage
 *
 * @input_name FIND_COLOR
 * @input_description Color to find
 * @input_type CvfdColor
 *
 * @parameter_name R_SENSIBILITY
 * @parameter_type int
 * @parameter_value 10
 * @parameter_description Sensibility with the red color
 *
 * @parameter_name G_SENSIBILITY
 * @parameter_type int
 * @parameter_value 10
 * @parameter_description Sensibility with the green color
 *
 * @parameter_name B_SENSIBILITY
 * @parameter_type int
 * @parameter_value 10
 * @parameter_description Sensibility with the blue color  
 * 
 * @output_name IMAGEOUT
 * @output_description Result image
 * @output_type CvfdImage
 *
 END*/
inline std::string CINT(int value)
{
  std::stringstream sstr;
  sstr << value;
  return(sstr.str());
}   
   
class CvfdFindColorRGB : public BufferedNode {
      
  //Input ID
  int m_imageInID;
  int m_findColorID;
  //Output ID
  int m_imageOutID;
  //Parameter
  int m_RSensibility;
  int m_GSensibility;
  int m_BSensibility;
      
public:
  CvfdFindColorRGB(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_imageInID = addInput("IMAGEIN");
    m_findColorID = addInput("FIND_COLOR");
         
    //add outputs
    m_imageOutID = addOutput("IMAGEOUT");
         
    //Initialize parameters         
    m_RSensibility = dereference_cast<int>(parameters.get("R_SENSIBILITY"));
    m_GSensibility = dereference_cast<int>(parameters.get("G_SENSIBILITY"));
    m_BSensibility = dereference_cast<int>(parameters.get("B_SENSIBILITY"));         
         
  }
      
  void calculate(int output_id, int count, Buffer &out)
  {           
    //Read the inputs         
    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);
    RCPtr<CvfdColor> findColorPtr = getInput(m_findColorID,count);         
    CvfdImage* image = new CvfdImage( cvGetSize(imagePtr->getImage()), IPL_DEPTH_8U, 1 );          
    unsigned char B,G,R;
    unsigned int x,y;
    int d=20;            
    for (x=0; x<(imagePtr->getImage())->width ; x++)
      {
	for (y=0; y<(imagePtr->getImage())->height ; y++)
	  {
	    B = ((uchar*)((imagePtr->getImage())->imageData + (imagePtr->getImage())->widthStep*y))[x*3];
	    G = ((uchar*)((imagePtr->getImage())->imageData + (imagePtr->getImage())->widthStep*y))[x*3+1];
	    R = ((uchar*)((imagePtr->getImage())->imageData + (imagePtr->getImage())->widthStep*y))[x*3+2];
               
	    if( ((findColorPtr->channel3()-m_BSensibility)<=B & B<=(findColorPtr->channel3()+m_BSensibility)) & 
		((findColorPtr->channel2()-m_GSensibility)<=G & G<=(findColorPtr->channel2()+m_GSensibility)) &
		((findColorPtr->channel1()-m_RSensibility)<=R & R<=(findColorPtr->channel1()+m_RSensibility)) )
	      {
		((uchar*)(image->getImage()->imageData + image->getImage()->widthStep*y))[x]=255;
	      }
	  }
      }                 
    out[count] = ObjectRef(image);
  }      
  NO_ORDER_NODE_SPEEDUP(CvfdFindColorRGB)
};
